@@ -65,7 +65,6 @@ sub new
 	bless $self, $class;
 	$self->{agent} ||= new WWW::GoodData::Agent ($root);
 	$self->{retries} ||= 3600;
-	$self->{agent}->{error_callback} = \&error_callback;
 	return $self;
 }
 
@@ -74,7 +73,8 @@ our %links;
 sub get_links
 {
 	my $self = shift;
-	my $root = (ref $_[0] and ref $_[0] ne 'HASH') ? shift : $root;
+
+	my $root = (ref $_[0] and ref $_[0] ne 'HASH') ? shift : new URI ($self->{agent}{root});
 	my @path = map { ref $_ ? $_ : { category => $_ } } @_;
 	my $link = shift @path;
 
@@ -207,9 +207,12 @@ sub login
 	my $self = shift;
 	my ($login, $password) = @_;
 
+	# We should really get a link here instead of mangling
+	# service root location...
+	my $root = new URI ($self->{agent}{root});
 	my $netloc = $root->host.':'.$root->port;
-	# Neither on the same address, not navigatable
 	$netloc =~ s/([^\.]*)/$1-di/;
+
 	$self->{agent}->credentials ($netloc,
 		'GoodData project data staging area', $login => $password);
 
@@ -365,12 +368,18 @@ sub export_report
 			report => $self->compute_report ($report) }}
 	);
 
+	# This is for new release, where location is finally set correctly;
+	$result = $result->{uri} if ref $result eq 'HASH';
+
 	# Trigger the export
 	my $exported = $self->poll (
 		sub { $self->{agent}->get ($result) },
-		sub { shift->{raw} ne 'null' }
+		sub { $_[0] and exists $_[0]->{raw} and $_[0]->{raw} ne 'null' }
 	) or die 'Timed out waiting for report to export';
 
+	# Follow the link
+	$exported = $self->{agent}->get ($exported->{uri}) if exists $exported->{uri};
+
 	# Gotten the correctly coded result?
 	return $exported->{raw} if $exported->{type} eq {
 		png => 'image/png',
@@ -471,7 +480,9 @@ sub upload
 		sub { shift->{taskStatus} !~ /^(RUNNING|PREPARED)$/ }
 	) or die 'Timed out waiting for integration to finish';
 
-	return $result->{taskStatus} ne 'ERROR';
+	return if $result->{taskStatus} eq 'OK';
+	warn 'Upload finished with warnings' if $result->{taskStatus} eq 'WARNING';
+	die 'Upload finished with '.$result->{taskStatus}.' status';
 }
 
 =item B<poll> BODY CONDITION
